@@ -207,6 +207,7 @@ class Cursor(DBAPICursor):
         self._query_id = None
         self._batch = list()
         self._rowcount = 0
+        self._iter_index = 0
 
     def _reset_state(self):
         """Reset state about the previous query in preparation for running another query"""
@@ -282,6 +283,8 @@ class Cursor(DBAPICursor):
         else:
             sql = operation % _escaper.escape_args(parameters)
 
+        _logger.info("{}".format(sql))
+
         client = self.connection.client
         self._query_id = client.prepareStatement(sql)
         client.executeStatement(self._query_id)
@@ -289,7 +292,6 @@ class Cursor(DBAPICursor):
         _logger.info("getting result metadata")
         buffer = client.getResultMetadata(self._query_id)
         buffer = BytesIO(buffer)
-        self.rows_count = 0
         self._rowcount, self._query_columns_description = get_query_columns_info(buffer)
         _logger.info("finished populating metadata")
         return
@@ -319,7 +321,7 @@ class Cursor(DBAPICursor):
             _logger.info("fetching next batch from fetch all")
             rows = self.fetch_batch()
             if rows is None:
-                return None
+                break
             self._data = self._data + rows
         return self._data
 
@@ -339,21 +341,28 @@ class Cursor(DBAPICursor):
         _logger.info("fetching all")
         return self._fetch_all()
 
-    def fetchmany(self, size=10000):
+    def fetchmany(self, size=None):
         _logger.info("fetching all from overriden method")
+        if size is None:
+            size = self.arraysize
         size = min(size, self.rowcount())
-        self.rows_count = 0
-        self._data = list()
-        while self.rows_count < size:
+        if self._data is None:
+            self._data = list()
+        while len(self._data) < size:
             _logger.info("fetching next batch from fetch many")
             _logger.info(len(self._data))
-            _logger.info("no  of rows accumulated ".format(self.rows_count))
+            _logger.info("no  of rows accumulated {}".format(self.rows_count))
             rows = self.fetch_batch()
             if rows is None:
-                return None
+                break
             self.rows_count = self.rows_count + len(rows)
-            self._data = self._data + rows
-        return self._data if len(self._data) == size else self._data[0:size]
+            print(self.rows_count)
+            self._data += rows
+        if len(self._data) == size:
+            return self._data
+        rows = self._data[self._iter_index:self._iter_index+size]
+        self._iter_index = size
+        return rows
 
     def fetchone(self):
         _logger.info("fetch One returning the batch itself which is limited by predefined no.of rows")
@@ -408,7 +417,7 @@ if __name__ == '__main__':
                          thrift_transport=None)
     cursor = Cursor(connection)
     cursor.execute("select * from date_dim;")
-    bdata = cursor.fetchone()
+    bdata = cursor.fetchall()
     try:
         # if len(bdata) < 1000:
         for row in bdata:
